@@ -28,13 +28,11 @@ fn main() {
         let parsed: Vec<Entry> = serde_json::from_reader(commands_file)
             .expect("Failed to parse RIOT_COMPILE_COMMANDS_JSON");
 
-        // We need to find a consensus list -- otherwise single modules like stdio_uart that define
-        // flags like -Wno-cast-function-type can throw things off. (It's not like the actual ABI
-        // compatibility should suffer from something like that, for any flags like enum packing
-        // need to be the same systemwide anyway for things to to go very wrong, and furthermore
-        // the flag should have been caught by the compile commands generation if it's not good for
-        // LLVM (maybe it is and bindgen is just behind) -- but at any rate, finding some consensus
-        // is a good idea here).
+        // We need to find a consensus list -- otherwise single modules like stdio_uart that
+        // defines anything odd for its own purpose can throw things off. (It's not like the actual
+        // ABI compatibility should suffer from them, for any flags like enum packing need to be
+        // the same systemwide anyway for things to to go very wrong) -- but at any rate, finding
+        // some consensus is to some extent necessary here).
         //
         // This is relatively brittle, but still better than the previous approach of just taking
         // the first entry.
@@ -160,6 +158,12 @@ fn main() {
     bindings
         .write_to_file(out_path.join("bindings.rs"))
         .expect("Couldn't write bindings!");
+    // Store for inspection for markers; see there
+    let mut bindgen_output = Vec::<u8>::new();
+    bindings
+        .write(Box::new(&mut bindgen_output))
+        .expect("String writing never fails");
+    let bindgen_output = std::str::from_utf8(&bindgen_output).expect("Rust source code is UTF-8");
 
     // Build a compile_commands.json, and run C2Rust
     //
@@ -462,4 +466,36 @@ fn main() {
             .as_bytes(),
         )
         .expect("Failed to write to toplevel_from_inline.rs");
+
+    // Some decisions of downstream crates need to depend on whether some feature is around in
+    // RIOT. For many things that's best checked on module level, but some minor items have no
+    // module to mark the feature, and checking for versions by numers is not fine-grained enough,
+    // so it's easiest to check for concrete strings in the bindgen output.
+    //
+    // These lead to `MARKER_foo=1` items emitted that are usable as `DEP_RIOT_SYS_MARKER_foo=1` by
+    // crates that explicitly `links = "riot-sys"`. They are stable in that they'll only go away in
+    // a breaking riot-sys version; downstream users likely stop using them earlier because they
+    // sooner or later stop supporting old RIOT versions.
+    //
+    // These markers are currently checked against bindgen's output, but could query any property
+    // that riot-sys has access to. The markers are defined in terms of some change having happened
+    // in RIOT; the way they are tested for can change. (In particular, when riot-sys stops
+    // supporting an older RIOT version, it can just always emit that marker).
+    //
+    // Crates building on this should preferably not alter their own APIs depending on these,
+    // because that would add extra (and hard-to-track) dimensions to them. If they can, they
+    // should provide a unified view and degrade gracefully. (For example, riot-wrappers has the
+    // unit `T` of the `phydat_unit_t` in its enum, but converts it to the generic unspecified unit
+    // on RIOT versions that don't have the T type yet -- at least for as long as it supports
+    // 2022.01).
+
+    let markers = [
+        // See https://github.com/RIOT-OS/RIOT/pull/17569, available after 2022.01
+        ("pub const UNIT_T:", "phydat_unit_t"),
+    ];
+    for (needle, name) in markers {
+        if bindgen_output.contains(needle) {
+            println!("cargo:MARKER_{}=1", name);
+        }
+    }
 }
