@@ -84,16 +84,8 @@
 #![allow(non_upper_case_globals)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
-// when experimenting with C2Rust generated extern functions, C library fn are pulled in and they
-// have stuff like `pub type iovec`
-#![feature(extern_types)]
-// eg. for irq_enable on arm
-#![feature(llvm_asm)]
 // Produced by c2rust for riscv32imac, eg. on hifive1 board (at least with TOOLCHAIN=llvm)
-#![feature(core_intrinsics)]
-// Produced in const functions on the rpi-pico and ek-lm4f120xl boards, eg. when casting around
-// LED_PORT.
-#![feature(const_raw_ptr_deref)]
+#![cfg_attr(target_arch = "riscv32", feature(core_intrinsics))]
 
 pub mod libc;
 
